@@ -538,9 +538,8 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         if utl != "":
             msn_object = papyon.p2p.MSNObject.parse(self._client, utl)
 
-        if len(cm_parts) < 3 or cm_parts[2] == '0':
-            cm = None
-        elif cm_parts[1] == 'Music' and cm_parts[2] == '1':
+        cm = None
+        if len(cm_parts) >= 6 and cm_parts[1] == 'Music' and cm_parts[2] == '1':
             cm = (cm_parts[4], cm_parts[5])
 
         eps = tree.findall("./EndpointData")
@@ -562,11 +561,11 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         contacts = self.__search_account(account, network_id)
         for contact in contacts:
             contact._add_flag(profile.ContactFlag.EXTENDED_PRESENCE_KNOWN)
+            contact._server_property_changed("end-points", end_points)
             contact._server_property_changed("msn-object", msn_object)
             contact._server_property_changed("current-media", cm)
             contact._server_property_changed("personal-message", pm)
             contact._server_property_changed("signature-sound", ss)
-            contact._server_property_changed("end-points", end_points)
 
     def _handle_UUN(self, command): # UBN acknowledgment
         pass
@@ -769,9 +768,16 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                 address_book.profile.display_name)
 
         contacts = address_book.contacts.group_by_domain()
+        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
+
+        for contact in address_book.contacts:
+            if (contact.memberships & mask & ~profile.Membership.FORWARD) == \
+                    (profile.Membership.ALLOW | profile.Membership.BLOCK):
+                logger.warning("Contact is on both Allow and Block list; " \
+                               "removing from Allow list (%s)" % contact.account)
+                contact._remove_membership(profile.Membership.ALLOW)
 
         payloads = ['<ml l="1">']
-        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
         for domain, contacts in contacts.iteritems():
             payloads[-1] += '<d n="%s">' % domain
             for contact in contacts:
