@@ -5,6 +5,7 @@
 # Copyright (C) 2005-2006 Ali Sabil <ali.sabil@gmail.com>
 # Copyright (C) 2006  Johann Prieur <johann.prieur@gmail.com>
 # Copyright (C) 2006  Ole André Vadla Ravnås <oleavr@gmail.com>
+# Copyright (C) 2010  Collabora Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,6 +31,8 @@ inside HTTP POST requests.
 The classes of this module are structured as follow:
 G{classtree BaseTransport}"""
 
+from gnet.proxy.factory import ProxyFactory
+
 import gnet
 import gnet.protocol
 import msnp
@@ -225,10 +228,7 @@ class DirectConnection(BaseTransport):
     def __init__(self, server, server_type=ServerType.NOTIFICATION, proxies={}):
         BaseTransport.__init__(self, server, server_type, proxies)
 
-        transport = gnet.io.TCPClient(server[0], server[1])
-        transport.connect("notify::status", self.__on_status_change)
-        transport.connect("error", self.__on_error)
-
+        transport = self._setup_transport(server[0], server[1], proxies)
         receiver = gnet.parser.DelimiterParser(transport)
         receiver.connect("received", self.__on_received)
 
@@ -242,6 +242,14 @@ class DirectConnection(BaseTransport):
 
     __init__.__doc__ = BaseTransport.__init__.__doc__
 
+    def _setup_transport(self, host, port, proxies):
+        transport = gnet.io.TCPClient(host, port)
+        if proxies:
+            transport = ProxyFactory(transport, proxies, 'direct')
+        transport.connect("notify::status", self.__on_status_change)
+        transport.connect("error", self.__on_error)
+        return transport
+
     ### public commands
 
     @property
@@ -292,7 +300,7 @@ class DirectConnection(BaseTransport):
 
     def __handle_ping_reply(self, command):
         timeout = int(command.arguments[0])
-        self.__png_timeout = gobject.timeout_add(timeout * 1000, self.enable_ping)
+        self.__png_timeout = gobject.timeout_add_seconds(timeout, self.enable_ping)
 
     ### callbacks
     def __on_status_change(self, transport, param):
@@ -350,28 +358,25 @@ class HTTPPollConnection(BaseTransport):
         self._target_server = server
         server = ("gateway.messenger.hotmail.com", 80)
         BaseTransport.__init__(self, server, server_type, proxies)
-        self._setup_transport()
+        self._setup_transport(server[0], server[1], proxies)
         
         self._command_queue = []
         self._waiting_for_response = False # are we waiting for a response
         self._session_id = None
         self.__error = False
 
-    def _setup_transport(self):
-        server = self.server
-        proxies = self.proxies
-        if 'http' in proxies:
-            transport = gnet.protocol.HTTP(server[0], server[1], proxies['http'])
-        else:
-            transport = gnet.protocol.HTTP(server[0], server[1])
-        transport.connect("response-received", self.__on_received)
-        transport.connect("request-sent", self.__on_sent)
-        transport.connect("error", self.__on_error)
+    def _setup_transport(self, host, port, proxies):
+        handles = []
+        transport = gnet.protocol.HTTP(host, port, proxies)
+        handles.append(transport.connect("response-received", self.__on_received))
+        handles.append(transport.connect("request-sent", self.__on_sent))
+        handles.append(transport.connect("error", self.__on_error))
+        self._transport_handles = handles
         self._transport = transport
 
     def establish_connection(self):
         logger.debug('<-> Connecting to %s:%d' % self.server)
-        self._polling_source_id = gobject.timeout_add(5000, self._poll)
+        self._polling_source_id = gobject.timeout_add_seconds(5, self._poll)
         self.emit("connection-success")
 
     def lose_connection(self):
@@ -386,6 +391,16 @@ class HTTPPollConnection(BaseTransport):
             self._target_server = server
         self.emit("connection-reset")
 
+    def change_gateway(self, server):
+        if self.server == server:
+            return
+        logger.debug('<-> Changing gateway to %s:%d' % server)
+        self.server = server
+        for handle in self._transport_handles:
+            self._transport.disconnect(handle)
+        self._transport.close()
+        self._setup_transport(server[0], server[1], self.proxies)
+
     def send_command(self, command, increment=True, callback=None, *cb_args):
         self._command_queue.append((command, increment, callback, cb_args))
         self._send_command()
@@ -431,12 +446,12 @@ class HTTPPollConnection(BaseTransport):
     
     def __on_error(self, transport, reason):
         self.__error = True
+        if reason == 403:
+            reason = TransportError.PROXY_FORBIDDEN
         self.emit("connection-lost", reason)
+        self.lose_connection()
         
     def __on_received(self, transport, http_response):
-        if http_response.status == 403:
-            self.emit("connection-lost", TransportError.PROXY_FORBIDDEN)
-            self.lose_connection()
         if 'X-MSN-Messenger' in http_response.headers:
             data = http_response.headers['X-MSN-Messenger'].split(";")
             for elem in data:
@@ -444,8 +459,7 @@ class HTTPPollConnection(BaseTransport):
                 if key == 'SessionID':
                     self._session_id = value
                 elif key == 'GW-IP':
-                    self.server = (value, self.server[1])
-                    self._setup_transport()
+                    self.change_gateway((value, self.server[1]))
                 elif key == 'Session'and value == 'close':
                     #self.lose_connection()
                     pass
